# Copyright (c) 2021 Red Hat, Inc.
#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import absolute_import

import asyncio
import typing


class ActorRequest(typing.NamedTuple):
    future: asyncio.Future
    method: str
    arguments: typing.Dict[str, typing.Any]


class ActorRequestQueue(object):

    def __init__(self, loop: asyncio.AbstractEventLoop, max_size=0):
        self._queue: asyncio.Queue = asyncio.Queue(maxsize=max_size)
        self._loop = loop

    def send_request(self,
                     method: str,
                     arguments: typing.Dict[str, typing.Any]):
        future = self._loop.create_future()
        request = ActorRequest(future=future,
                               method=method,
                               arguments=arguments)
        self._queue.put_nowait(request)
        return future

    async def receive_request(self):
        return await self._queue.get()


def create_request_queue(loop: asyncio.AbstractEventLoop, max_size=0) \
        -> ActorRequestQueue:
    return ActorRequestQueue(loop=loop, max_size=max_size)
